extern crate pkg_config;
extern crate cc;

use std::env;

fn main() {
    // Has the user requested static linking?
    let mut wants_static = cfg!(feature = "static") || env::var("GEOGRAPHICLIB_STATIC").is_ok();

    // Try to find GeographicLib using pkg-config
    if !try_pkgconfig(wants_static) {
        // It probably fails on Windows, but we can try to find it manually (and link statically)
        if try_windows() {
            wants_static = true;
        }
        // If it fails, we can't find GeographicLib
        else {
            panic!("GeographicLib not found");
        }
    }

    // Source files for this crate
    let src = [
        "src/RhumbLine.cpp",
        "src/Geodesic.cpp",
    ];

    // Starts the build
    let mut build = cc::Build::new();

    // Set the flag to build a static or shared library
    build.define("GEOGRAPHICLIB_SHARED_LIB", if wants_static { "0" } else { "1" });

    // Add the source files
    build.files(src.iter());

    // Add the include path
    if let Ok(include_path) = env::var("GEOGRAPHICLIB_INCLUDE_PATH") {
        build.include(include_path);
    }

    // Compile the library
    build.compile("geographiclib-sys");

    // Tell cargo to recompile if the source files change
    for file in src.iter() {
        println!("cargo:rerun-if-changed={}", file);
    }
}

// Tries to find GeographicLib using pkg-config
fn try_pkgconfig(wants_static: bool) -> bool {
    let mut pkg = pkg_config::Config::new();
    pkg.statik(wants_static);

    if let Ok(lib) = pkg.probe("geographiclib") {
        if let Some(path) = lib.include_paths.get(0) {
            println!("cargo:include={}", path.display());
        }
        if let Some(path) = lib.link_paths.get(0) {
            println!("cargo:root={}", path.display());
        }
        return true;
    }
    false
}

// Tries to find GeographicLib on Windows, hard-coded
fn try_windows() -> bool {
    if let Ok(path) = glob::glob("C:\\Program Files*\\GeographicLib*") {
        for entry in path {
            if let Ok(entry) = entry {
                println!("cargo:rustc-link-search=native={}", entry.join("lib").display());
                println!("cargo:rustc-link-lib=static=GeographicLib");
                println!("cargo:include={}", entry.join("include").display());

                // Set the include path for cc::Build
                env::set_var("GEOGRAPHICLIB_INCLUDE_PATH", entry.join("include").display().to_string());

                return true;
            }
        }
    }
    false
}